@@ -1,6 +1,5 @@
 /*
- * Copyright (c) 2012-2017 Meltytech, LLC
- * Author: Dan Dennedy <dan@dennedy.org>
+ * Copyright (c) 2012-2020 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -123,7 +122,9 @@ private slots:
     void on_videoQualitySpinner_valueChanged(int vq);
 
     void on_audioQualitySpinner_valueChanged(int aq);
-    
+
+    void on_parallelCheckbox_clicked(bool checked);
+
 private:
     enum {
         RateControlAverage = 0,
