@@ -64,6 +64,7 @@ TimelineDock::TimelineDock(QWidget *parent) :
     m_quickView.rootContext()->setContextProperty("multitrack", &m_model);
     m_quickView.setResizeMode(QQuickWidget::SizeRootObjectToView);
     m_quickView.setClearColor(palette().window().color());
+    m_quickView.quickWindow()->setPersistentSceneGraph(false);
 
     connect(&m_model, SIGNAL(modified()), this, SLOT(clearSelectionIfInvalid()));
     connect(&m_model, &MultitrackModel::inserted, this, &TimelineDock::onInserted, Qt::QueuedConnection);
@@ -111,12 +112,12 @@ Mlt::ClipInfo *TimelineDock::getClipInfo(int trackIndex, int clipIndex)
     return result;
 }
 
-Mlt::Producer *TimelineDock::producerForClip(int trackIndex, int clipIndex)
+Mlt::Producer TimelineDock::producerForClip(int trackIndex, int clipIndex)
 {
-    Mlt::Producer* result = nullptr;
+    Mlt::Producer result;
     Mlt::ClipInfo* info = getClipInfo(trackIndex, clipIndex);
     if (info) {
-        result = new Mlt::Producer(info->producer);
+        result = Mlt::Producer(info->producer);
         delete info;
     }
     return result;
@@ -522,7 +523,7 @@ void TimelineDock::onShowFrame(const SharedFrame& frame)
 {
     if (m_ignoreNextPositionChange) {
         m_ignoreNextPositionChange = false;
-    } else if (MLT.isMultitrack()) {
+    } else if (MLT.isMultitrack() && m_position != frame.get_position()) {
         m_position = frame.get_position();
         emit positionChanged();
     }
@@ -530,7 +531,7 @@ void TimelineDock::onShowFrame(const SharedFrame& frame)
 
 void TimelineDock::onSeeked(int position)
 {
-    if (MLT.isMultitrack()) {
+    if (MLT.isMultitrack() && m_position != position) {
         m_position = position;
         emit positionChanged();
     }
@@ -564,9 +565,8 @@ void TimelineDock::remove(int trackIndex, int clipIndex)
         return;
     }
     Q_ASSERT(trackIndex >= 0 && clipIndex >= 0);
-    Mlt::Producer* clip = producerForClip(trackIndex, clipIndex);
-    if (clip) {
-        delete clip;
+    Mlt::Producer clip = producerForClip(trackIndex, clipIndex);
+    if (clip.is_valid()) {
         MAIN.undoStack()->push(
             new Timeline::RemoveCommand(m_model, trackIndex, clipIndex));
     }
@@ -581,9 +581,9 @@ void TimelineDock::lift(int trackIndex, int clipIndex)
         return;
     }
     if (trackIndex < 0 || clipIndex < 0) return;
-    QScopedPointer<Mlt::Producer> clip(producerForClip(trackIndex, clipIndex));
-    if (clip) {
-        if (clip->is_blank())
+    Mlt::Producer clip(producerForClip(trackIndex, clipIndex));
+    if (clip.is_valid()) {
+        if (clip.is_blank())
             return;
         MAIN.undoStack()->push(
             new Timeline::LiftCommand(m_model, trackIndex, clipIndex));
@@ -677,6 +677,10 @@ void TimelineDock::selectMultitrack()
 
 void TimelineDock::copyClip(int trackIndex, int clipIndex)
 {
+    if (trackIndex < 0)
+        trackIndex = currentTrack();
+    if (clipIndex < 0)
+        clipIndex = clipIndexAtPlayhead(trackIndex);
     Q_ASSERT(trackIndex >= 0 && clipIndex >= 0);
     QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
     if (info) {
@@ -802,7 +806,7 @@ void TimelineDock::onRowsRemoved(const QModelIndex& parent, int first, int last)
                 else if (i.y() > last)
                     newSelection << QPoint(i.x(), i.y() - n);
             }
-        }            
+        }
         setSelection(newSelection);
         if (!parent.isValid())
             model()->reload(true);
@@ -848,6 +852,38 @@ void TimelineDock::onProducerModified()
     emitSelectedChanged(QVector<int>() << MultitrackModel::NameRole);
 }
 
+void TimelineDock::replace(int trackIndex, int clipIndex, const QString& xml)
+{
+    if (xml.isEmpty() && !MLT.isClip() && !MLT.savedProducer()) {
+        showStatusMessage(tr("There is nothing in the Source player."));
+        return;
+    }
+    if (!m_model.trackList().count() || MAIN.isSourceClipMyProject())
+        return;
+    if (trackIndex < 0)
+        trackIndex = currentTrack();
+    if (isTrackLocked(trackIndex)) {
+        pulseLockButtonOnTrack(trackIndex);
+        return;
+    }
+    if (clipIndex < 0)
+        clipIndex = clipIndexAtPlayhead(trackIndex);
+    Mlt::Producer producer(producerForClip(trackIndex, clipIndex));
+    if (producer.is_valid() && producer.type() == tractor_type) {
+        showStatusMessage(tr("You cannot replace a transition."));
+        return;
+    }
+    if (MLT.isSeekableClip() || MLT.savedProducer() || !xml.isEmpty()) {
+        Q_ASSERT(trackIndex >= 0 && clipIndex >= 0);
+        QString xmlToUse = !xml.isEmpty()? xml
+            : MLT.XML(MLT.isClip()? nullptr : MLT.savedProducer());
+        MAIN.undoStack()->push(
+            new Timeline::ReplaceCommand(m_model, trackIndex, clipIndex, xmlToUse));
+    } else if (!MLT.isSeekableClip()) {
+        emit showStatusMessage(kNonSeekableWarning);
+    }
+}
+
 void TimelineDock::setTrackName(int trackIndex, const QString &value)
 {
     MAIN.undoStack()->push(
@@ -885,6 +921,20 @@ bool TimelineDock::moveClip(int fromTrack, int toTrack, int clipIndex, int posit
         if (m_updateCommand)
             m_updateCommand->setPosition(toTrack, clipIndex, position);
     } else {
+        // Check for locked tracks
+        auto trackDelta = toTrack - fromTrack;
+        for (const auto& clip : selection()) {
+            auto trackIndex = clip.y() + trackDelta;
+            if (isTrackLocked(clip.y())) {
+                pulseLockButtonOnTrack(clip.y());
+                return false;
+            }
+            if (isTrackLocked(trackIndex)) {
+                pulseLockButtonOnTrack(trackIndex);
+                return false;
+            }
+        }
+
         // Workaround bug #326 moving clips between tracks stops allowing drag-n-drop
         // into Timeline, which appeared with Qt 5.6 upgrade.
         emit clipMoved(fromTrack, toTrack, clipIndex, position, ripple);
@@ -958,17 +1008,18 @@ bool TimelineDock::trimClipIn(int trackIndex, int clipIndex, int oldClipIndex, i
         }
         clipIndex = m_model.trimClipIn(trackIndex, clipIndex, delta, ripple, Settings.timelineRippleAllTracks());
 
-        // Update duration in properties for image clip.
-        QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
-        if (info && MLT.isImageProducer(info->producer) && !info->producer->get_int(kShotcutSequenceProperty))
-            emit imageDurationChanged();
-
         m_trimDelta += delta;
         m_trimCommand.reset(new Timeline::TrimClipInCommand(m_model, trackIndex, oldClipIndex, m_trimDelta, ripple, false));
         if (m_updateCommand && m_updateCommand->trackIndex() == trackIndex && m_updateCommand->clipIndex() == clipIndex)
             m_updateCommand->setPosition(trackIndex, clipIndex, m_updateCommand->position() + delta);
     }
     else return false;
+
+    // Update duration in properties
+    QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
+    if (info && !info->producer->get_int(kShotcutSequenceProperty))
+        emit durationChanged();
+
     return true;
 }
 
@@ -1005,17 +1056,18 @@ bool TimelineDock::trimClipOut(int trackIndex, int clipIndex, int delta, bool ri
         }
         m_model.trimClipOut(trackIndex, clipIndex, delta, ripple, Settings.timelineRippleAllTracks());
 
-        // Update duration in properties for image clip.
-        QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
-        if (info && MLT.isImageProducer(info->producer) && !info->producer->get_int(kShotcutSequenceProperty))
-            emit imageDurationChanged();
-
         m_trimDelta += delta;
         m_trimCommand.reset(new Timeline::TrimClipOutCommand(m_model, trackIndex, clipIndex, m_trimDelta, ripple, false));
         if (m_updateCommand && m_updateCommand->trackIndex() == trackIndex && m_updateCommand->clipIndex() == clipIndex)
             m_updateCommand->setPosition(trackIndex, clipIndex,-1);
     }
     else return false;
+
+    // Update duration in properties
+    QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
+    if (info && !info->producer->get_int(kShotcutSequenceProperty))
+        emit durationChanged();
+
     return true;
 }
 
@@ -1283,3 +1335,75 @@ void TimelineDock::onTransitionAdded(int trackIndex, int clipIndex, int position
     // Select the transition.
     setSelection(QList<QPoint>() << QPoint(command->getTransitionIndex(), trackIndex));
 }
+
+class FindProducersByHashParser : public Mlt::Parser
+{
+private:
+    QString m_hash;
+    QList<Mlt::Producer> m_producers;
+
+public:
+    FindProducersByHashParser(const QString& hash)
+        : Mlt::Parser()
+        , m_hash(hash)
+    {}
+
+    QList<Mlt::Producer>& producers() { return m_producers; }
+
+    int on_start_filter(Mlt::Filter*) { return 0; }
+    int on_start_producer(Mlt::Producer* producer) {
+        if (producer->is_cut() && MAIN.getHash(producer->parent()) == m_hash)
+            m_producers << Mlt::Producer(producer);
+        return 0;
+    }
+    int on_end_producer(Mlt::Producer*) { return 0; }
+    int on_start_playlist(Mlt::Playlist*) { return 0; }
+    int on_end_playlist(Mlt::Playlist*) { return 0; }
+    int on_start_tractor(Mlt::Tractor*) { return 0; }
+    int on_end_tractor(Mlt::Tractor*) { return 0; }
+    int on_start_multitrack(Mlt::Multitrack*) { return 0; }
+    int on_end_multitrack(Mlt::Multitrack*) { return 0; }
+    int on_start_track() { return 0; }
+    int on_end_track() { return 0; }
+    int on_end_filter(Mlt::Filter*) { return 0; }
+    int on_start_transition(Mlt::Transition*) { return 0; }
+    int on_end_transition(Mlt::Transition*) { return 0; }
+};
+
+void TimelineDock::replaceClipsWithHash(const QString& hash, Mlt::Producer& producer)
+{
+    FindProducersByHashParser parser(hash);
+    parser.start(*model()->tractor());
+    auto n = parser.producers().size();
+    if (n > 1)
+        MAIN.undoStack()->beginMacro(tr("Replace %1 timeline clips").arg(n));
+    for (auto& clip : parser.producers()) {
+        int trackIndex = -1;
+        int clipIndex = -1;
+        // lookup the current track and clip index by UUID
+        QScopedPointer<Mlt::ClipInfo> info(MAIN.timelineClipInfoByUuid(clip.get(kUuidProperty), trackIndex, clipIndex));
+
+        if (trackIndex >= 0 && clipIndex >= 0) {
+            int in = clip.get_in();
+            int out = clip.get_out();
+
+            // Factor in a transition left of the clip.
+            QScopedPointer<Mlt::ClipInfo> info2(getClipInfo(trackIndex, clipIndex - 1));
+            if (info2 && info2->producer && info2->producer->is_valid()
+                      && info2->producer->get(kShotcutTransitionProperty)) {
+                in -= info2->frame_count;
+            }
+            // Factor in a transition right of the clip.
+            info2.reset(getClipInfo(trackIndex, clipIndex + 1));
+            if (info2 && info2->producer && info2->producer->is_valid()
+                      && info2->producer->get(kShotcutTransitionProperty)) {
+                out += info2->frame_count;
+            }
+            Util::applyCustomProperties(producer, *info->producer, in, out);
+
+            replace(trackIndex, clipIndex, MLT.XML(&producer));
+        }
+    }
+    if (n > 1)
+        MAIN.undoStack()->endMacro();
+}
