@@ -487,7 +487,7 @@ void GLWidget::mouseMoveEvent(QMouseEvent* event)
 {
     QQuickWidget::mouseMoveEvent(event);
     if (event->isAccepted()) return;
-    if (event->modifiers() == Qt::ShiftModifier && m_producer) {
+    if (event->modifiers() == (Qt::ShiftModifier | Qt::AltModifier) && m_producer) {
         emit seekTo(m_producer->get_length() * event->x() / width());
         return;
     }
@@ -973,6 +973,11 @@ void FrameRenderer::showFrame(Mlt::Frame frame)
     }
     emit frameDisplayed(m_displayFrame);
 
+    if (m_imageRequested) {
+        m_imageRequested = false;
+        emit imageReady();
+    }
+
     m_semaphore.release();
 }
 
