@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2019 Meltytech, LLC
+ * Copyright (c) 2011-2020 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -276,7 +276,7 @@ void Controller::onJackStarted(int position)
     if (m_producer) {
         m_producer->set_speed(1);
         m_producer->seek(position);
-        refreshConsumer();
+        Controller::refreshConsumer();
     }
 }
 
@@ -468,10 +468,9 @@ bool Controller::saveXML(const QString& filename, Service* service, bool withRel
         c.set("time_format", "clock");
         c.set("no_meta", 1);
         c.set("store", "shotcut");
-        if (withRelativePaths) {
-            c.set("root", fi.absolutePath().toUtf8().constData());
-            c.set("no_root", 1);
-        }
+        c.set("root", withRelativePaths? fi.absolutePath().toUtf8().constData() : "");
+        c.set("root", "");
+        c.set("no_root", 1);
         c.set("title", QString("Shotcut version ").append(SHOTCUT_VERSION).toUtf8().constData());
         c.connect(s);
         c.start();
@@ -1034,9 +1033,12 @@ QUuid Controller::ensureHasUuid(Mlt::Properties& properties) const
     return newUid;
 }
 
-void Controller::copyFilters(Producer& fromProducer, Producer& toProducer)
+void Controller::copyFilters(Producer& fromProducer, Producer& toProducer, bool fromClipboard)
 {
+    int in = fromProducer.get(kFilterInProperty)? fromProducer.get_int(kFilterInProperty) : fromProducer.get_in();
+    int out = fromProducer.get(kFilterOutProperty)? fromProducer.get_int(kFilterOutProperty) : fromProducer.get_out();
     int count = fromProducer.filter_count();
+    
     for (int i = 0; i < count; i++) {
         QScopedPointer<Mlt::Filter> fromFilter(fromProducer.filter(i));
         if (fromFilter && fromFilter->is_valid() && !fromFilter->get_int("_loader") && fromFilter->get("mlt_service")) {
@@ -1044,6 +1046,13 @@ void Controller::copyFilters(Producer& fromProducer, Producer& toProducer)
             if (toFilter.is_valid()) {
                 toFilter.inherit(*fromFilter);
                 toProducer.attach(toFilter);
+
+                if (!fromClipboard) {
+                    toFilter.set(kFilterInProperty, fromFilter->get_in() - in);
+                    if (fromFilter->get_out() != out) {
+                        toFilter.set(kFilterOutProperty, fromFilter->get_out() - fromFilter->get_in());
+                    }
+                }
             }
         }
     }
@@ -1067,60 +1076,70 @@ void Controller::pasteFilters(Mlt::Producer* producer)
                       : nullptr;
     if (targetProducer) {
         int j = targetProducer->filter_count();
-        copyFilters(*m_filtersClipboard, *targetProducer);
+        copyFilters(*m_filtersClipboard, *targetProducer, true);
+        adjustFilters(*targetProducer, j);
+    }
+}
 
-        // Adjust filters.
-        bool changed = false;
-        int n = targetProducer->filter_count();
-        for (; j < n; j++) {
-            QScopedPointer<Mlt::Filter> filter(targetProducer->filter(j));
+void Controller::adjustFilters(Producer& producer, int index)
+{
+    bool changed = false;
+    int in = producer.get(kFilterInProperty)? producer.get_int(kFilterInProperty) : producer.get_in();
+    int out = producer.get(kFilterOutProperty)? producer.get_int(kFilterOutProperty): producer.get_out();
+    int n = producer.filter_count();
 
-            if (filter && filter->is_valid()) {
-                QString filterName = filter->get(kShotcutFilterProperty);
-                int in = targetProducer->get(kFilterInProperty)? targetProducer->get_int(kFilterInProperty) : targetProducer->get_in();
-                int out = targetProducer->get(kFilterOutProperty)? targetProducer->get_int(kFilterOutProperty): targetProducer->get_out();
-                if (filterName.startsWith("fadeIn") && !filter->get(kShotcutAnimInProperty)) {
-                    // Convert legacy fadeIn filters.
-                    filter->set(kShotcutAnimInProperty, filter->get_length());
-                }
-                else if (filterName.startsWith("fadeOut") && !filter->get(kShotcutAnimOutProperty)) {
-                    // Convert legacy fadeIn filters.
-                    filter->set(kShotcutAnimOutProperty, filter->get_length());
-                }
-                if (!filter->get_int("_loader")) {
-                    filter->set_in_and_out(in, out);
-                    changed = true;
+    for (; index < n; index++) {
+        QScopedPointer<Mlt::Filter> filter(producer.filter(index));
 
-                    if (filterName == "fadeOutBrightness") {
-                        const char* key = filter->get_int("alpha") != 1? "alpha" : "level";
-                        filter->clear(key);
-                        filter->anim_set(key, 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                        filter->anim_set(key, 0, filter->get_length() - 1);
-                    } else if (filterName == "fadeOutMovit") {
-                        filter->clear("opacity");
-                        filter->anim_set("opacity", 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty), 0, mlt_keyframe_smooth);
-                        filter->anim_set("opacity", 0, filter->get_length() - 1);
-                    } else if (filterName == "fadeOutVolume") {
-                        filter->clear("level");
-                        filter->anim_set("level", 0, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                        filter->anim_set("level", -60, filter->get_length() - 1);
-                    } else if (filter->get_int(kShotcutAnimOutProperty) > 0) {
-                        // Update simple keyframes.
-                        QmlMetadata* meta = MAIN.filterController()->metadataForService(filter.data());
-                        if (meta && meta->keyframes()) {
-                            foreach (QString name, meta->keyframes()->simpleProperties()) {
-                                const char* propertyName = name.toUtf8().constData();
-                                if (!filter->get_animation(propertyName))
-                                    // Cause a string property to be interpreted as animated value.
-                                    filter->anim_get_double(propertyName, 0, filter->get_length());
-                                Mlt::Animation animation = filter->get_animation(propertyName);
-                                if (animation.is_valid()) {
-                                    int n = animation.key_count();
-                                    if (n > 1) {
-                                        animation.set_length(filter->get_length());
-                                        animation.key_set_frame(n - 2, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                                        animation.key_set_frame(n - 1, filter->get_length() - 1);
-                                    }
+        if (filter && filter->is_valid()) {
+            QString filterName = filter->get(kShotcutFilterProperty);
+            if (filterName.startsWith("fadeIn") && !filter->get(kShotcutAnimInProperty)) {
+                // Convert legacy fadeIn filters.
+                filter->set(kShotcutAnimInProperty, filter->get_length());
+            }
+            else if (filterName.startsWith("fadeOut") && !filter->get(kShotcutAnimOutProperty)) {
+                // Convert legacy fadeIn filters.
+                filter->set(kShotcutAnimOutProperty, filter->get_length());
+            }
+            if (!filter->get_int("_loader")) {
+                int filterIn = in;
+                int filterOut = out;
+                if (filter->get(kFilterInProperty))
+                    filterIn += filter->get_int(kFilterInProperty);
+                if (filter->get(kFilterOutProperty))
+                    filterOut = qMin(filterIn + filter->get_int(kFilterOutProperty), out);
+                filter->set_in_and_out(filterIn, filterOut);
+                changed = true;
+
+                if (filterName == "fadeOutBrightness") {
+                    const char* key = filter->get_int("alpha") != 1? "alpha" : "level";
+                    filter->clear(key);
+                    filter->anim_set(key, 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                    filter->anim_set(key, 0, filter->get_length() - 1);
+                } else if (filterName == "fadeOutMovit") {
+                    filter->clear("opacity");
+                    filter->anim_set("opacity", 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty), 0, mlt_keyframe_smooth);
+                    filter->anim_set("opacity", 0, filter->get_length() - 1);
+                } else if (filterName == "fadeOutVolume") {
+                    filter->clear("level");
+                    filter->anim_set("level", 0, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                    filter->anim_set("level", -60, filter->get_length() - 1);
+                } else if (filter->get_int(kShotcutAnimOutProperty) > 0) {
+                    // Update simple keyframes.
+                    QmlMetadata* meta = MAIN.filterController()->metadataForService(filter.data());
+                    if (meta && meta->keyframes()) {
+                        foreach (QString name, meta->keyframes()->simpleProperties()) {
+                            const char* propertyName = name.toUtf8().constData();
+                            if (!filter->get_animation(propertyName))
+                                // Cause a string property to be interpreted as animated value.
+                                filter->anim_get_double(propertyName, 0, filter->get_length());
+                            Mlt::Animation animation = filter->get_animation(propertyName);
+                            if (animation.is_valid()) {
+                                int n = animation.key_count();
+                                if (n > 1) {
+                                    animation.set_length(filter->get_length());
+                                    animation.key_set_frame(n - 2, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                                    animation.key_set_frame(n - 1, filter->get_length() - 1);
                                 }
                             }
                         }
@@ -1128,9 +1147,9 @@ void Controller::pasteFilters(Mlt::Producer* producer)
                 }
             }
         }
-        if (changed)
-            refreshConsumer();
     }
+    if (changed)
+        MLT.refreshConsumer();
 }
 
 void Controller::setSavedProducer(Mlt::Producer* producer)
