@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2019 Meltytech, LLC
+ * Copyright (c) 2012-2020 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ Player::Player(QWidget *parent)
     , m_zoomToggleFactor(Settings.playerZoom() == 0.0f? 1.0f : Settings.playerZoom())
     , m_pauseAfterOpen(false)
     , m_monitorScreen(-1)
-    , m_currentTransport(0)
+    , m_currentTransport(nullptr)
 {
     setObjectName("Player");
     Mlt::Controller::singleton();
@@ -193,15 +193,15 @@ Player::Player(QWidget *parent)
     m_durationLabel = new QLabel(this);
     m_durationLabel->setToolTip(tr("Total Duration"));
     m_durationLabel->setText(" / 00:00:00:00");
-    m_durationLabel->setFixedWidth(m_positionSpinner->width());
+    m_durationLabel->setFixedWidth(m_positionSpinner->width() - 20);
     m_inPointLabel = new QLabel(this);
     m_inPointLabel->setText("--:--:--:--");
     m_inPointLabel->setToolTip(tr("In Point"));
-    m_inPointLabel->setFixedWidth(m_inPointLabel->width());
+    m_inPointLabel->setFixedWidth(m_positionSpinner->width() - 20);
     m_selectedLabel = new QLabel(this);
     m_selectedLabel->setText("--:--:--:--");
     m_selectedLabel->setToolTip(tr("Selected Duration"));
-    m_selectedLabel->setFixedWidth(m_selectedLabel->width());
+    m_selectedLabel->setFixedWidth(m_positionSpinner->width() - 30);
     toolbar->addWidget(m_positionSpinner);
     toolbar->addWidget(m_durationLabel);
     toolbar->addWidget(spacer);
@@ -225,10 +225,10 @@ Player::Player(QWidget *parent)
         tr("Zoom 25%"), this, SLOT(onZoomTriggered()))->setData(0.25f);
     m_zoomMenu->addAction(
         QIcon::fromTheme("zoom-out", QIcon(":/icons/oxygen/32x32/actions/zoom-out")),
-        tr("Zoom 50%"), this, SLOT(onZoomTriggered()))->setData(0.5f);;
+        tr("Zoom 50%"), this, SLOT(onZoomTriggered()))->setData(0.5f);
     m_zoomMenu->addAction(
         QIcon::fromTheme("zoom-original", QIcon(":/icons/oxygen/32x32/actions/zoom-original")),
-        tr("Zoom 100%"), this, SLOT(onZoomTriggered()))->setData(1.0f);;
+        tr("Zoom 100%"), this, SLOT(onZoomTriggered()))->setData(1.0f);
     m_zoomMenu->addAction(
         QIcon::fromTheme("zoom-in", QIcon(":/icons/oxygen/32x32/actions/zoom-in")),
         tr("Zoom 200%"), this, SLOT(onZoomTriggered()))->setData(2.0f);
@@ -471,7 +471,7 @@ bool Player::event(QEvent* event)
 void Player::play(double speed)
 {
     // Start from beginning if trying to start at the end.
-    if (m_position >= m_duration - 1) {
+    if (m_position >= m_duration - 1 && !MLT.isMultitrack()) {
         emit seeked(m_previousIn);
         m_position = m_previousIn;
     }
@@ -700,7 +700,7 @@ void Player::updateSelection()
 
 void Player::onInChanged(int in)
 {
-    if (in != m_previousIn) {
+    if (in != m_previousIn && in >= 0) {
         int delta = in - MLT.producer()->get_in();
         MLT.setIn(in);
         emit inChanged(delta);
@@ -711,7 +711,7 @@ void Player::onInChanged(int in)
 
 void Player::onOutChanged(int out)
 {
-    if (out != m_previousOut) {
+    if (out != m_previousOut && out >= 0) {
         int delta = out - MLT.producer()->get_out();
         MLT.setOut(out);
         emit outChanged(delta);
@@ -937,7 +937,7 @@ void Player::setPauseAfterOpen(bool pause)
 
 Player::TabIndex Player::tabIndex() const
 {
-    return (TabIndex)m_tabs->currentIndex();
+    return TabIndex(m_tabs->currentIndex());
 }
 
 //----------------------------------------------------------------------------
@@ -945,22 +945,22 @@ Player::TabIndex Player::tabIndex() const
 
 static inline float IEC_dB ( float fScale )
 {
-	float dB = 0.0f;
+    float dB = 0.0f;
 
-	if (fScale < 0.025f)	    // IEC_Scale(-60.0f)
-		dB = (fScale / 0.0025f) - 70.0f;
-	else if (fScale < 0.075f)	// IEC_Scale(-50.0f)
-		dB = (fScale - 0.025f) / 0.005f - 60.0f;
-	else if (fScale < 0.15f)	// IEC_Scale(-40.0f)
-		dB = (fScale - 0.075f) / 0.0075f - 50.0f;
-	else if (fScale < 0.3f)		// IEC_Scale(-30.0f)
-		dB = (fScale - 0.15f) / 0.015f - 40.0f;
-	else if (fScale < 0.5f)		// IEC_Scale(-20.0f)
-		dB = (fScale - 0.3f) / 0.02f - 30.0f;
-	else /* if (fScale < 1.0f)	// IED_Scale(0.0f)) */
-		dB = (fScale - 0.5f) / 0.025f - 20.0f;
+    if (fScale < 0.025f)	    // IEC_Scale(-60.0f)
+        dB = (fScale / 0.0025f) - 70.0f;
+    else if (fScale < 0.075f)	// IEC_Scale(-50.0f)
+        dB = (fScale - 0.025f) / 0.005f - 60.0f;
+    else if (fScale < 0.15f)	// IEC_Scale(-40.0f)
+        dB = (fScale - 0.075f) / 0.0075f - 50.0f;
+    else if (fScale < 0.3f)		// IEC_Scale(-30.0f)
+        dB = (fScale - 0.15f) / 0.015f - 40.0f;
+    else if (fScale < 0.5f)		// IEC_Scale(-20.0f)
+        dB = (fScale - 0.3f) / 0.02f - 30.0f;
+    else /* if (fScale < 1.0f)	// IED_Scale(0.0f)) */
+        dB = (fScale - 0.5f) / 0.025f - 20.0f;
 
-	return (dB > -0.001f && dB < 0.001f ? 0.0f : dB);
+    return (dB > -0.001f && dB < 0.001f ? 0.0f : dB);
 }
 
 void Player::onVolumeChanged(int volume)
