@@ -88,6 +88,7 @@ AvformatProducerWidget::AvformatProducerWidget(QWidget *parent)
     , m_userDefinedCaption(false)
 {
     ui->setupUi(this);
+    ui->timelineDurationText->setFixedWidth(ui->durationSpinBox->width());
     ui->filenameLabel->setFrame(true);
     Util::setColorsToHighlight(ui->filenameLabel, QPalette::Base);
     if (Settings.playerGPU())
@@ -123,8 +124,12 @@ Mlt::Producer* AvformatProducerWidget::newProducer(Mlt::Profile& profile)
         p = new Mlt::Producer(profile, s.toUtf8().constData());
         p->set(kShotcutProducerProperty, "avformat");
     }
-    if (p->is_valid())
+    if (p->is_valid()) {
         p->set("video_delay", double(ui->syncSlider->value()) / 1000);
+        if (ui->pitchCheckBox->checkState() == Qt::Checked) {
+            m_producer->set("warp_pitch", 1);
+        }
+    }
     return p;
 }
 
@@ -134,6 +139,20 @@ void AvformatProducerWidget::setProducer(Mlt::Producer* p)
     emit producerChanged(p);
 }
 
+void AvformatProducerWidget::updateDuration()
+{
+    if (m_producer->get(kFilterInProperty) && m_producer->get(kFilterOutProperty)) {
+        auto duration = m_producer->get_int(kFilterOutProperty) - m_producer->get_int(kFilterInProperty) + 1;
+        ui->timelineDurationLabel->show();
+        ui->timelineDurationText->setText(m_producer->frames_to_time(duration));
+        ui->timelineDurationText->show();
+    } else {
+        ui->timelineDurationLabel->hide();
+        ui->timelineDurationLabel->setText(QString());
+        ui->timelineDurationText->hide();
+    }
+}
+
 void AvformatProducerWidget::keyPressEvent(QKeyEvent* event)
 {
     if (ui->speedSpinBox->hasFocus() &&
@@ -223,7 +242,7 @@ void AvformatProducerWidget::recreateProducer()
 {
     Mlt::Producer* p = newProducer(MLT.profile());
     p->pass_list(*m_producer, "audio_index, video_index, force_aspect_ratio,"
-                 "video_delay, force_progressive, force_tff, set.force_full_luma, color_range,"
+                 "video_delay, force_progressive, force_tff, set.force_full_luma, color_range, warp_pitch,"
                  kAspectRatioNumerator ","
                  kAspectRatioDenominator ","
                  kShotcutHashProperty ","
@@ -273,8 +292,19 @@ void AvformatProducerWidget::onFrameDecoded()
     ui->filenameLabel->setToolTip(resource);
     ui->notesTextEdit->setPlainText(QString::fromUtf8(m_producer->get(kCommentProperty)));
     ui->durationSpinBox->setValue(m_producer->get_length());
+    updateDuration();
     m_recalcDuration = false;
     ui->speedSpinBox->setValue(warpSpeed);
+    if (warpSpeed == 1.0) {
+        ui->pitchCheckBox->setEnabled(false);
+    } else {
+        ui->pitchCheckBox->setEnabled(true);
+    }
+    if (m_producer->get_int("warp_pitch") == 1) {
+        ui->pitchCheckBox->setCheckState(Qt::Checked);
+    } else {
+        ui->pitchCheckBox->setCheckState(Qt::Unchecked);
+    }
     ui->rangeComboBox->setEnabled(true);
 
     // populate the track combos
@@ -508,6 +538,7 @@ void AvformatProducerWidget::onFrameDecoded()
 void AvformatProducerWidget::on_resetButton_clicked()
 {
     ui->speedSpinBox->setValue(1.0);
+    ui->pitchCheckBox->setCheckState(Qt::Unchecked);
     Mlt::Producer* p = newProducer(MLT.profile());
     ui->durationSpinBox->setValue(m_defaultDuration);
     ui->syncSlider->setValue(0);
@@ -607,10 +638,27 @@ void AvformatProducerWidget::on_speedSpinBox_editingFinished()
         return;
     if (ui->speedSpinBox->value() == GetSpeedFromProducer(producer()))
         return;
+    if (ui->speedSpinBox->value() == 1.0) {
+        ui->pitchCheckBox->setEnabled(false);
+    } else {
+        ui->pitchCheckBox->setEnabled(true);
+    }
     m_recalcDuration = true;
     recreateProducer();
 }
 
+void AvformatProducerWidget::on_pitchCheckBox_stateChanged(int state)
+{
+    if (!m_producer)
+        return;
+    if (state == Qt::Unchecked) {
+        m_producer->set("warp_pitch", 0);
+    } else {
+        m_producer->set("warp_pitch", 1);
+    }
+    emit modified();
+}
+
 void AvformatProducerWidget::on_syncSlider_valueChanged(int value)
 {
     double delay = double(value) / 1000.0;
@@ -755,7 +803,7 @@ void AvformatProducerWidget::convert(TranscodeDialog& dialog)
             args << "-y" << filename;
             FfmpegJob* job = new FfmpegJob(filename, args, false);
             job->setLabel(tr("Convert %1").arg(Util::baseName(filename)));
-            job->setPostJobAction(new FilePropertiesPostJobAction(resource, filename));
+            job->setPostJobAction(new ConvertReplacePostJobAction(resource, filename, MAIN.getHash(*m_producer)));
             JOBS.add(job);
         }
     }
@@ -780,19 +828,29 @@ void AvformatProducerWidget::on_reverseButton_clicked()
         QStringList ffmpegArgs;
         QString nameFilter;
         QString ffmpegSuffix = "mov";
+        int in = -1;
 
         ffmpegArgs << "-loglevel" << "verbose";
         ffmpegArgs << "-i" << resource;
         ffmpegArgs << "-max_muxing_queue_size" << "9999";
         // set trim options
-        if (m_producer->get(kFilterInProperty))
-            ffmpegArgs << "-ss" << QString::fromLatin1(m_producer->get_time(kFilterInProperty, mlt_time_clock)).replace(',', '.');
-        else
+        if (m_producer->get(kFilterInProperty)) {
+            in = m_producer->get_int(kFilterInProperty);
+            int ss = qMax(0, in - qRound(m_producer->get_fps() * 15.0));
+            auto s = QString::fromLatin1(m_producer->frames_to_time(ss, mlt_time_clock));
+            ffmpegArgs << "-ss" << s.replace(',', '.');
+        } else {
             ffmpegArgs << "-ss" << QString::fromLatin1(m_producer->get_time("in", mlt_time_clock)).replace(',', '.').replace(',', '.');
-        if (m_producer->get(kFilterOutProperty))
-            ffmpegArgs << "-to" << QString::fromLatin1(m_producer->get_time(kFilterOutProperty, mlt_time_clock)).replace(',', '.');
-        else
+        }
+        if (m_producer->get(kFilterOutProperty)) {
+            int out = m_producer->get_int(kFilterOutProperty);
+            int to = qMin(m_producer->get_playtime() - 1, out + qRound(m_producer->get_fps() * 15.0));
+            in = to - out - 1;
+            auto s = QString::fromLatin1(m_producer->frames_to_time(to, mlt_time_clock));
+            ffmpegArgs << "-to" << s.replace(',', '.');
+        } else {
             ffmpegArgs << "-to" << QString::fromLatin1(m_producer->get_time("out", mlt_time_clock)).replace(',', '.');
+        }
         // transcode all streams except data, subtitles, and attachments
         ffmpegArgs << "-map" << "0:V?" << "-map" << "0:a?" << "-map_metadata" << "0" << "-ignore_unknown";
         if (ui->rangeComboBox->currentIndex())
@@ -894,7 +952,22 @@ void AvformatProducerWidget::on_reverseButton_clicked()
             MeltJob* meltJob = new MeltJob(filename, meltArgs,
                 m_producer->get_int("meta.media.frame_rate_num"), m_producer->get_int("meta.media.frame_rate_den"));
             meltJob->setLabel(tr("Reverse %1").arg(Util::baseName(resource)));
-            meltJob->setPostJobAction(new ReverseFilePostJobAction(resource, filename, tmpFileName));
+
+            if (m_producer->get(kMultitrackItemProperty)) {
+                QString s = QString::fromLatin1(m_producer->get(kMultitrackItemProperty));
+                QVector<QStringRef> parts = s.splitRef(':');
+                if (parts.length() == 2) {
+                    int clipIndex = parts[0].toInt();
+                    int trackIndex = parts[1].toInt();
+                    QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
+                    if (!uuid.isNull()) {
+                        meltJob->setPostJobAction(new ReverseReplacePostJobAction(resource, filename, tmpFileName, uuid.toByteArray(), in));
+                        JOBS.add(meltJob);
+                        return;
+                    }
+                }
+            }
+            meltJob->setPostJobAction(new ReverseOpenPostJobAction(resource, filename, tmpFileName));
             JOBS.add(meltJob);
         }
     }
@@ -987,3 +1060,8 @@ void AvformatProducerWidget::on_filenameLabel_editingFinished()
         emit modified();
     }
 }
+
+void AvformatProducerWidget::on_convertButton_clicked()
+{
+    on_actionFFmpegConvert_triggered();
+}
