@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2019 Meltytech, LLC
+ * Copyright (c) 2012-2020 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ ImageProducerWidget::ImageProducerWidget(QWidget *parent) :
     m_defaultDuration(-1)
 {
     ui->setupUi(this);
-    Util::setColorsToHighlight(ui->filenameLabel);
+    Util::setColorsToHighlight(ui->filenameLabel, QPalette::Base);
 }
 
 ImageProducerWidget::~ImageProducerWidget()
@@ -58,15 +58,24 @@ void ImageProducerWidget::setProducer(Mlt::Producer* p)
     AbstractProducerWidget::setProducer(p);
     if (m_defaultDuration == -1)
         m_defaultDuration = m_producer->get_length();
-    QString s;
-    if (m_producer->get(kShotcutResourceProperty))
-        s = QString::fromUtf8(m_producer->get(kShotcutResourceProperty));
-    else {
-        s = QString::fromUtf8(m_producer->get("resource"));
+    QString resource;
+    if (m_producer->get(kShotcutResourceProperty)) {
+        resource = QString::fromUtf8(m_producer->get(kShotcutResourceProperty));
+    } else {
+        resource = QString::fromUtf8(m_producer->get("resource"));
         p->set("ttl", 1);
     }
-    ui->filenameLabel->setText(ui->filenameLabel->fontMetrics().elidedText(QDir::toNativeSeparators(s), Qt::ElideLeft, width() - 40));
+    QString name = Util::baseName(resource);
+    QString caption = m_producer->get(kShotcutCaptionProperty);
+    if (caption.isEmpty()) {
+        caption = name;
+        m_producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
+    }
+    ui->filenameLabel->setText(ui->filenameLabel->fontMetrics().elidedText(caption, Qt::ElideLeft, width() - 30));
     updateDuration();
+    resource = QDir::toNativeSeparators(resource);
+    ui->filenameLabel->setToolTip(resource);
+    m_producer->set(kShotcutDetailProperty, resource.toUtf8().constData());
     ui->resolutionLabel->setText(QString("%1x%2").arg(p->get("meta.media.width")).arg(p->get("meta.media.height")));
     ui->aspectNumSpinBox->blockSignals(true);
     if (p->get(kAspectRatioNumerator) && p->get(kAspectRatioDenominator)) {
@@ -132,12 +141,11 @@ void ImageProducerWidget::recreateProducer()
         }
     }
     Mlt::Producer* p = newProducer(MLT.profile());
+    if (resource.startsWith("qimage:") || resource.startsWith("pixbuf:"))
+        m_producer->set("resource", resource.mid(resource.indexOf(':') + 1).toUtf8().constData());
     p->pass_list(*m_producer, "force_aspect_ratio," kAspectRatioNumerator ", resource, " kAspectRatioDenominator
         ", begin, ttl," kShotcutResourceProperty ", autolength, length," kShotcutSequenceProperty ", " kPlaylistIndexProperty
         ", " kCommentProperty);
-    resource = p->get("resource");
-    if (resource.startsWith("qimage:") || resource.startsWith("pixbuf:"))
-        p->set("resource", resource.mid(resource.indexOf(':') + 1).toUtf8().constData());
     Mlt::Controller::copyFilters(*m_producer, *p);
     if (m_producer->get(kMultitrackItemProperty)) {
         emit producerChanged(p);
@@ -308,3 +316,22 @@ void ImageProducerWidget::on_actionSetFileDate_triggered()
     FileDateDialog dialog(resource, producer(), this);
     dialog.exec();
 }
+
+void ImageProducerWidget::on_filenameLabel_editingFinished()
+{
+    if (m_producer) {
+        auto caption = ui->filenameLabel->text();
+        if (caption.isEmpty()) {
+            if (m_producer->get(kShotcutResourceProperty))
+                caption = QString::fromUtf8(m_producer->get(kShotcutResourceProperty));
+            else
+                caption = QString::fromUtf8(m_producer->get("resource"));
+            caption = Util::baseName(caption);
+            ui->filenameLabel->setText(caption);
+            m_producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
+        } else {
+            m_producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
+        }
+        emit modified();
+    }
+}
